#!/usr/bin/env python3
# -*- coding: utf-8 -*-

MY_CONSTANT = 12124


def my_new_first_function(arg1, arg2):
    """This is my doc string of things"""
    ans = arg1 * arg2
    return ans


a = 10
b = 20

if __name__ == '__main__':
    print(function1(5, 6))

    print(function2(5, 6))
